"""CLI configuration management."""

import json
import logging
import os
import subprocess
from datetime import datetime
from functools import cache
from pathlib import Path
from typing import Literal, TypedDict

import jwt
import typer
from tomlkit import TOMLDocument, table
from tomlkit.exceptions import TOMLKitError
from tomlkit.items import Table
from tomlkit.toml_file import TOMLFile

from . import err_console
from .const import AUTH_FILENAME, CONFIG_DIR, JWT_SUPPORTED_ALGORITHMS, SETTINGS_FILENAME
from .oauth2.config import settings
from .oauth2.flow import RefreshFlow
from .oauth2.token import AuthJWT, OAuthToken

logger = logging.getLogger(__name__)

config_dir_location = Path.home() / CONFIG_DIR
auth_file_location = config_dir_location / AUTH_FILENAME
settings_file_location = config_dir_location / SETTINGS_FILENAME

jwks_client = jwt.PyJWKClient(settings["jwks_url"])


class AuthConfig(TypedDict):
    """Auth config file contents."""

    version: Literal["0.1"]
    oauth_token: OAuthToken


def save_auth_token(auth_token: OAuthToken):
    """Save authentication token."""

    auth_config: AuthConfig = {"version": "0.1", "oauth_token": auth_token}

    config_dir_location.mkdir(exist_ok=True)
    with auth_file_location.open("w") as f:
        json.dump(auth_config, f)

    auth_file_location.chmod(0o600)  # set secure file permissions


def parse_auth_token(auth_token: OAuthToken) -> AuthJWT:
    """Parse auth token into an AuthJWT object."""
    jwt_token = auth_token["access_token"]
    signing_key = jwks_client.get_signing_key_from_jwt(jwt_token)

    return jwt.decode(
        jwt_token,
        signing_key.key,
        algorithms=JWT_SUPPORTED_ALGORITHMS,
        audience=settings["audience"],
        leeway=15,
    )


def delete_auth_token():
    """Delete authentication token."""
    auth_file_location.unlink(missing_ok=True)


def refresh_auth_token(refresh_token: str) -> OAuthToken:
    """Refresh the authentication token."""

    new_auth_token = RefreshFlow().perform_refresh(refresh_token)

    save_auth_token(new_auth_token)

    return new_auth_token


def load_auth_token() -> OAuthToken | None:
    """Load authentication token from file."""

    # Check if we have an external token provider
    if auth_token_provider := os.environ.get("SLICES_AUTH_TOKEN_PROVIDER"):
        try:
            return json.loads(subprocess.check_output(auth_token_provider))  # noqa: S603
        except subprocess.CalledProcessError as e:
            err_console.print(f"[bold red]Could not load auth token from provider {auth_token_provider} :x:")
            err_console.print(f"[red]{e.output}")

    if not auth_file_location.exists():
        return None

    with Path(auth_file_location).open() as f:
        auth_file_contents = json.load(f)

    if not auth_file_contents:
        logger.error("Could not load authentication token")
        return None

    if auth_file_contents["version"] != "0.1":
        logger.warning("Unexpected auth file version: %s", auth_file_contents["version"])

    auth_token: OAuthToken = auth_file_contents["oauth_token"]

    if "refresh_token" in auth_token and datetime.fromtimestamp(auth_token["expires_at"]) < datetime.now():
        auth_token = refresh_auth_token(auth_token["refresh_token"])

    return auth_token


@cache
def _load_settings() -> TOMLDocument:
    config_dir_location.mkdir(exist_ok=True)

    try:
        if settings_file_location.exists():
            settings_file = TOMLFile(settings_file_location)
            return settings_file.read()
    except TOMLKitError as e:
        err_console.print(f"[bold red]Could not read settings file {settings_file_location}")
        err_console.print(e)
        raise typer.Exit(13) from e

    return TOMLDocument()


def _save_settings(config: TOMLDocument):
    config_dir_location.mkdir(exist_ok=True)
    settings_file = TOMLFile(settings_file_location)

    settings_file.write(config)


def set_config_string(name: str, value: str | None, section: str | None = None):
    """Set config value."""

    settings = _load_settings()

    if section:
        if section not in settings:
            section_table = table()
            settings.add(section, section_table)
        else:
            section_table = settings[section]
            if not isinstance(section_table, Table):
                raise ValueError(f"Expected '{section}' to be a table.")

        if value:
            section_table[name] = value
        else:
            del section_table[name]
    else:
        settings[name] = value

    _save_settings(settings)


def get_config_string(name: str, default=None, section: str | None = None) -> str | None:
    """Get config value."""

    settings = _load_settings()

    if section:
        if section in settings:
            section_item = settings[section]
            if not isinstance(section_item, Table):
                raise ValueError(f"Expected {section} to be a table.")
            return section_item.get(name, default)
        return default
    return settings.get(name, default)
