"""Slices CLI authentication."""

import json
import os
from datetime import datetime
from enum import Enum
from typing import Annotated, Optional

import humanize
import typer
import yaml
from pytz import UTC
from rich.table import Table

from . import console, err_console
from .config import set_config_string
from .context import Context
from .utils import friendly_datetime

project_app = typer.Typer(help="Projects", no_args_is_help=True)


class ListFormat(Enum):
    """List format options."""

    table = "table"
    json = "json"
    yaml = "yaml"


@project_app.command(help="Show projects of which you are a member", rich_help_panel="Commands")
def list_projects(
    ctx: Context,
    show_all: Annotated[bool, typer.Option(help="Also show expired projects")] = False,
    show_details: Annotated[bool, typer.Option(help="Show project IDs")] = False,
    format: Annotated[ListFormat, typer.Option("--format", "-f", help="Output format.")] = "table",  # type: ignore
):
    """List projects."""
    projects = ctx.obj.api.projects_api.get_projects(include_expired=show_all)

    if format == ListFormat.json:
        console.print_json(projects.model_dump_json(exclude_unset=True, by_alias=True))
        return
    if format == ListFormat.yaml:
        console.print(yaml.safe_dump(projects.model_dump(mode="json", exclude_unset=True, by_alias=True)))
        return

    columns = [
        "Name",
        "Role",
        "Created At",
        "Expires At",
    ]

    if show_details:
        columns.insert(0, "ID")

    table = Table(*columns, title=f"Projects for {ctx.obj.get_logged_in_userinfo('preferred_username')}")
    for project in projects.projects:
        row = [
            project.name,
            project.role,
            friendly_datetime(project.created_at),
            friendly_datetime(project.expires_at) if project.expires_at else "-",
        ]
        if show_details:
            row.insert(0, project.id)

        table.add_row(*row)

    console.print(table)


class ShowProjectFormat(Enum):
    """Show project format Options."""

    text = "text"
    json = "json"
    yaml = "yaml"


@project_app.command(help="Show the current project", rich_help_panel="Commands")
def show(
    ctx: Context,
    format: Annotated[ShowProjectFormat, typer.Option("--format", "-f", help="Output format.")] = "text",  # type: ignore
    show_members: Annotated[
        bool,
        typer.Option(
            "--show-members/--no-show-members",
            help="Show members. (Default: --show-members)",
            show_default=False,
        ),
    ] = True,
):
    """Show the current project, and it's members."""
    current_project_id = ctx.obj.get_current_project_id()

    if not current_project_id:
        err_console.print("[red]No project set. :x:")
        raise typer.Exit

    project = ctx.obj.api.projects_api.get_project(current_project_id)

    if format == ShowProjectFormat.text:
        console.print(f"[bold]The current project is '{project.name}'.")
        console.print(f":ticket: You are a {project.role} in this project.")

        if project.expires_at:
            humanized_delta = humanize.precisedelta(
                project.expires_at - datetime.now(UTC),
                minimum_unit="days",
                format="%0.0f",
            )
            if project.expires_at < datetime.now(UTC):
                console.print(
                    f"[red]:clock10: It expired on {friendly_datetime(project.expires_at)}" f" ({humanized_delta} ago)."
                )
            else:
                console.print(
                    f":clock10: It expires on {friendly_datetime(project.expires_at)}" f" (in {humanized_delta})"
                )
        else:
            console.print("[green]:clock10: It never expires.")

    project_members = []
    if show_members:
        project_members = ctx.obj.other_users_info.get_project_members(current_project_id)

        if format == ShowProjectFormat.text:
            table = Table("User ID", "Name", "Email", "Role", title=f"Members of project {project.name}")

            for member in project_members:
                try:
                    extra_user_info = ctx.obj.api.users_api.get_user(member.user_id)
                except Exception as e:  # noqa: E722
                    err_console.print(f"Warning: problem fetching extra user info: {e}")
                    extra_user_info = None
                name = f"{extra_user_info.first_name} {extra_user_info.last_name}" if extra_user_info else "unknown"
                email = extra_user_info.email if extra_user_info else "unknown"
                table.add_row(member.user_id, name, email, member.role)
            console.print(table)

    if format in (ShowProjectFormat.json, ShowProjectFormat.yaml):
        obj = project.model_dump(mode="json", by_alias=True, exclude_unset=True)
        if show_members:
            obj = {
                **obj,
                "members": [m.model_dump(mode="json", by_alias=True, exclude_unset=True) for m in project_members],
            }

        if format == ShowProjectFormat.yaml:
            console.print(yaml.safe_dump(obj))
        elif format == ShowProjectFormat.json:
            console.print_json(json.dumps(obj))
        return


@project_app.command(help="Set the current project", rich_help_panel="Commands")
def use(
    ctx: Context,
    name: Annotated[str, typer.Argument(help="The ID or the name of the project that you want to set as default.")],
):
    """Set the current project."""

    if "SLICES_PROJECT_ID" in os.environ:
        err_console.print("[red]Project is set via the SLICES_PROJECT_ID environment variable. :x:")
        raise typer.Exit(1)

    projects = ctx.obj.api.projects_api.get_projects()

    project = None
    for candidate_project in projects.projects:
        if name in (candidate_project.id, candidate_project.name):
            project = candidate_project
            break

    if not project:
        err_console.print(f"[red]Could not find project '{name}'. :x:")
        err_console.print(
            f"The following projects are available: {', '.join(project.name for project in projects.projects)}"
        )
        raise typer.Exit(1)

    set_config_string("default", project.id, "project")

    console.print(
        f"[green]The current project was set to '[bold]{project.name}[/]', "
        f"""in which you are a [bold]{project.role}[/] and which {
        f'expires on {friendly_datetime(project.expires_at)}'
        if project.expires_at
        else 'never expires'}."""
    )


@project_app.command(help="Clear the current project", rich_help_panel="Advanced Commands")
def clear():
    """Clear the current project."""

    if "SLICES_PROJECT_ID" in os.environ:
        err_console.print("[red]Project is set via the SLICES_PROJECT_ID environment variable. :x:")
        raise typer.Exit(1)

    set_config_string("default", None, "project")
    console.print("Cleared current project.")


@project_app.command(help="Get a JWT project token", rich_help_panel="Advanced Commands")
def token(
    ctx: Context,
    name: Annotated[
        Optional[str], typer.Argument(help="The ID or the name of the project that you want to set as default.")
    ] = None,
):
    """Get a JWT project token."""

    if name:
        projects = ctx.obj.api.projects_api.get_projects()

        project = None
        for candidate_project in projects.projects:
            if name in (candidate_project.id, candidate_project.name):
                project = candidate_project
                break

        if not project:
            err_console.print(f"[red]Could not find project '{name}'. :x:")
            err_console.print(
                f"The following projects are available: {', '.join(project.name for project in projects.projects)}"
            )
            raise typer.Exit(1)

        current_project_id = project.id
    else:
        current_project_id = ctx.obj.get_current_project_id()

        if not current_project_id:
            err_console.print("[red]No project set. :x:")
            raise typer.Exit(1)

    project_token = ctx.obj.api.get_project_jwt(current_project_id)

    print(project_token)
