"""CLI Utils."""

import json
import re
from datetime import datetime
from json import JSONDecodeError
from pathlib import Path
from typing import Any

import typer

PUBLIC_SSH_KEY_LOCATIONS = ["~/.ssh/id_ed25519.pub", "~/.ssh/id_ecdsa.pub", "~/.ssh/id_rsa.pub"]


def load_public_ssh_keys() -> list[str]:
    """Load SSH keys from well known locations."""
    public_ssh_keys = []
    for ssh_key_location in PUBLIC_SSH_KEY_LOCATIONS:
        path = Path(ssh_key_location).expanduser()
        if path.exists():
            public_ssh_keys.append(path.read_text())

    return public_ssh_keys


local_timezone = datetime.now().astimezone().tzinfo


def friendly_datetime(dt: datetime, *, include_seconds: bool = False):
    """Format datetime in local timezone in a human-readable manner."""
    # return dt.astimezone(local_timezone).strftime("%Y-%m-%d %H:%M")
    return dt.astimezone(local_timezone).strftime(
        "%Y-%m-%d %H:%M %Z" if not include_seconds else "%Y-%m-%d %H:%M:%S %Z"
    )


def datetime_with_local_timezone(dt: datetime):
    """Make naive datetime into a datetime with local timezone."""
    if dt.tzinfo is not None:
        raise ValueError("Expected naive datetime object.")
    return dt.astimezone(local_timezone)


def get_error_detail(error_body: Any | None):
    """Get detail from error response."""
    if not isinstance(error_body, str):
        return None
    try:
        error_body_contents = json.loads(error_body)
    except JSONDecodeError:
        return error_body

    error_details = []

    if "detail" in error_body_contents:
        if isinstance(error_body_contents["detail"], list):
            for detail in error_body_contents["detail"]:
                if "msg" in detail:
                    error_details.append(detail["msg"])
                else:
                    error_details.append(str(detail))
        else:
            error_details.append(error_body_contents["detail"])

    if error_details:
        res = []
        for detail in error_details:
            if isinstance(detail, str):
                res.append(detail)
            elif isinstance(detail, dict | list):
                res.append(json.dumps(detail, indent=4))
            else:
                res.append(str(detail))
        return "\n".join(res)

    return "No details available"


# TYPEID_PARTS = 3
# TYPEID_SUFFIX_LENGTH = 26
#
#
# def is_type_id(value: str, required_type: str | None = None):
#     """Is given string a valid TypeID."""
#     if value.count("_") != TYPEID_PARTS - 1:
#         return False
#
#     type_, domain, suffix = value.split("_")
#     return len(suffix) == TYPEID_SUFFIX_LENGTH and (type_ is None or type_ == required_type)


def is_type_id(s: str, type: str | None = None, domain: str | None = None):
    """Is given string a valid TypeID."""
    type_pattern = type if type else r"[a-z]{1,6}"
    domain_pattern = domain if domain else r"[a-z0-9-.]{2,56}"
    suffix_pattern = r"[01234567][0123456789abcdefghjkmnpqrstvwxyz]{25}"
    pattern = re.compile(f"^{type_pattern}_{domain_pattern}_{suffix_pattern}$")

    return pattern.match(s)


def is_experiment_id(s: str):
    """Is given string a valid experiment TypeID."""
    return is_type_id(s, type="exp")


def is_project_id(s: str):
    """Is given string a valid project TypeID."""
    return is_type_id(s, type="proj")


def validate_type_id(value: str, *, required_type: str | None = None):
    """Validate TypeID."""

    if value is None:
        return None

    if not is_type_id(value):
        raise typer.BadParameter("invalid ID")

    type_, domain, suffix = value.split("_")
    if type_ is not None and type_ != required_type:
        raise typer.BadParameter(f"invalid ID type. Expected {required_type!r}, got {type_!r}")

    return value
